/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

package com.sunlabs.archsim.fleet.ships.stride;

// import java.util.Iterator;
// import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.engine.Director;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.gasp.State;

public class StrideTest extends CompositeEntity {

	StrideTestSource testSource;
	Stride stride;
	StrideTestSink testSink;	
	
	public StrideTest(String name) {
		super(name);
		
		// make components
		testSource= new StrideTestSource();
		addMember(testSource);
		stride= new Stride("testStride", null);
		addMember(stride);
		testSink= new StrideTestSink();
		addMember(testSink);
		
		// make the states
		State startValueState= new State("startValueState", this);
		State ackStartValueState = new State("ackValueState, this");
		State stepSizeState= new State("stepSizeState", this);
		State ackStepSizeState= new State("ackStepSizeState", this);
		State stepCountState= new State("stepCountState", this);
		State ackStepCountState= new State("ackStepCountState, this");
		State doneState= new State("doneState", this);
		State ackDoneState= new State("ackDoneState", this);
		State nextState= new State("nextState", this);
		State ackNextState= new State("ackNextState", this);
		State counterState= new State("counterState", this);
		State ackCounterState= new State("ackCounterState", this);
		
		// make connections
		startValueState.attach(testSource.getTerminal(StrideTestSource.START_VALUE));
		startValueState.attach(stride.getTerminal(StrideDefaults.START_VALUE));
		
		ackStartValueState.attach(testSource.getTerminal(StrideTestSource.ACK_START_VALUE));
		ackStartValueState.attach(stride.getTerminal(StrideDefaults.ACK_START_VALUE));
		
		stepSizeState.attach(testSource.getTerminal(StrideTestSource.STEP_SIZE));
		stepSizeState.attach(stride.getTerminal(StrideDefaults.STEP));

		ackStepSizeState.attach(testSource.getTerminal(StrideTestSource.ACK_STEP_SIZE));
		ackStepSizeState.attach(stride.getTerminal(StrideDefaults.ACK_STEP));

		stepCountState.attach(testSource.getTerminal(StrideTestSource.NO_STEPS));
		stepCountState.attach(stride.getTerminal(StrideDefaults.STEP_COUNT));

		ackStepCountState.attach(testSource.getTerminal(StrideTestSource.ACK_NO_STEPS));
		ackStepCountState.attach(stride.getTerminal(StrideDefaults.ACK_STEP_COUNT));

		doneState.attach(testSource.getTerminal(StrideTestSource.DONE));
		doneState.attach(stride.getTerminal(StrideDefaults.DONE));

		ackDoneState.attach(testSource.getTerminal(StrideTestSource.ACK_DONE));
		ackDoneState.attach(stride.getTerminal(StrideDefaults.ACK_DONE));
		
		nextState.attach(testSink.getTerminal(StrideTestSink.NEXT));
		nextState.attach(stride.getTerminal(StrideDefaults.NEXT));

		ackNextState.attach(testSink.getTerminal(StrideTestSink.ACK_NEXT));
		ackNextState.attach(stride.getTerminal(StrideDefaults.ACK_NEXT));

		counterState.attach(testSink.getTerminal(StrideTestSink.COUNTER));
		counterState.attach(stride.getTerminal(StrideDefaults.COUNTER));

		ackCounterState.attach(testSink.getTerminal(StrideTestSink.ACK_COUNTER));
		ackCounterState.attach(stride.getTerminal(StrideDefaults.ACK_COUNTER));
		
		// activate initially-active states
		ackStartValueState.setInitiallyActive(null);
		ackStepSizeState.setInitiallyActive(null);
		ackStepCountState.setInitiallyActive(null);
		doneState.setInitiallyActive(null);
		
		ackCounterState.setInitiallyActive(null);
		nextState.setInitiallyActive(null);
		
		
	} // StrideTest constructor
	
	public static void main(String[] args) {
		System.out.println("StrideTest main");

		
		Director director= Director.getInstance();
		
		StrideTest strideTest= new StrideTest("JUStrideTest");
		
		// run self-check
		Boolean selfCheck= strideTest.selfCheck();
		Boolean testsPassed= false;
		// run the rest of the test only if self-check passed
		if (selfCheck) {
			director.reset();
			// run for long enough to go through all the tests
			director.run(new Delay(10000));
			testsPassed= strideTest.testCheck();
			
		}
		
		if (!selfCheck) {
			System.out.println("Self-check failed.");
		}
		else if (!testsPassed){
			System.out.println("Tests failed.");
		}
		else {
			System.out.println("Tests passed.");
		}
	}
	
	/**
	 * Has the test passed?
	 * Return true if yes, false if not.
	 * 
	 * @return boolean indicating whether the test has passed
	 */
	public boolean testCheck() {
		
		// get simulated results
		List<Data> results= testSink.getResults();
		// expected results
		LinkedList<Data> reference= new LinkedList<Data>();		
		/*
		// compute expected results

		for (int cycle= 0; cycle<testSource.noCycles; cycle++) {
			int i; int counter;
			for (counter= (Integer)testSource.startValue[cycle].value, i= 0;
					i<(Integer)testSource.noSteps[cycle].value; 
					counter+=(Integer)testSource.stepSize[cycle].value,i++ ) {
				reference.addLast(new Data(counter));
			} // one cycle
		} // for cycles		
		*/

		// compute expected results
		for (int cycle= 0; cycle<testSource.noCycles; cycle++) {
			int startCount= (Integer)testSource.startValue[cycle].value;
			boolean startCountValid= testSource.startValue[cycle].valid;
			int stepSize= (Integer)testSource.stepSize[cycle].value;
			boolean stepSizeValid= testSource.stepSize[cycle].valid;
			int noSteps= (Integer)testSource.noSteps[cycle].value;
			boolean noStepsValid= testSource.noSteps[cycle].valid;
			
			// is either of the counters invalid, is the number of steps non-positive,
			// is the start value out of range for the counter
			// then we have an invalid count setup
			boolean stepValid= startCountValid 
					&& stepSizeValid 
					&& noStepsValid
					&& (noSteps > 0) 
					&& (startCount < StrideDefaults.MAX_COUNTER) 
					&& (startCount > StrideDefaults.MIN_COUNTER);
			
			int counter= startCount;
			int step= 0; 
			boolean done= false;
			do {
				if (stepValid) {
					// add the result
					reference.addLast(new Data(counter));
				}
				else {
					// this is it: the cycle is done, data is invalid
					done= true;
					reference.addLast(new Data(false));
				}
				// get ready for the next step
				step++;
				done= done || step >= noSteps;
				if (!done) {
					// more steps to go through, see whether the counter would run out of bounds
					if ((stepSize>0 && (StrideDefaults.MAX_COUNTER-stepSize) < counter)
						|| (stepSize<0 && (StrideDefaults.MIN_COUNTER-stepSize) > counter)) {
						// out of bound, indicate thet the next step will be invalis
						stepValid= false;
					}
					else {
						// next step OK, get the next counter value
						counter= counter + stepSize;
					} 
				}
			} while (!done);
			
			
		} // for cycle
		
		
		/* display simulated results */
		/*
		System.out.println();
		System.out.println("Stride test, simulated resluts: ");

		Iterator li= results.iterator();
		int count= 0;
		while (li.hasNext()) {
			count++;
			System.out.println("Result: " + count + ": " + li.next());
		}
		System.out.println("Total count: " + count);
		System.out.println();
		
		// display computed results
		System.out.println();
		System.out.println("Stride test, computed resluts: ");
		li= reference.iterator();
		count= 0;
		while (li.hasNext()) {
			count++;
			System.out.println("Result: " + count + ": " + li.next());
		}
		System.out.println("Total count: " + count);
		System.out.println();
		*/
		
		// compare results
		if (results.equals(reference)) {
			return true;
		}
		else {
			return false;
		}
	} // testCheck
	
	
} // StrideTest
